package ben.namespace;

import java.util.ArrayList;
import android.graphics.Bitmap;
import android.graphics.Color;

//useful :http://kiwi.cs.dal.ca/~dparks/CornerDetection/algorithms.htm

public class CornerFinder 
{
	//settings for algorithm
	
	private static final int precision = 3;
	private static final int[] difs = {2};
	private static final int minDif = 100;
	private static final int sectors = 20;
	private static final int acceptableDistance = 20;
	
	public static ArrayList<Point> findCorners(Bitmap bitmap)
	/*
	 * Find corners in passed bitmap using three step approach:
	 * 		1. Find 'cornerness' of each point
	 * 		2. Find points with maximum 'cornerness' in each sector
	 * 		3. Remove corners too close together
	 */
	{
		Point[][] pointWithMeasure = findCornernessMeasure(bitmap);		//step 1
		ArrayList<Point> localMaxes = findLocalMax(pointWithMeasure);	//step 2
		ArrayList<Point> finalCorners = findMaximallySuppressed(localMaxes);	//step 3
		return finalCorners;
	}
	
	private static Point[][] findCornernessMeasure(Bitmap startBitmap)
	/*
	 * Calculates 'cornerness' of each pixel
	 * Instead of calculating for every pixel, scales the image to be precision times smaller
	 */
	{
		//scale image
		Bitmap bitmap = Bitmap.createScaledBitmap(startBitmap, startBitmap.getWidth()/precision, startBitmap.getHeight()/precision, false);
		
		//create array of points in new scaled image
		Point[][] corners = new Point[bitmap.getWidth()][bitmap.getHeight()];
		
		//loop through points, initialize
		for (int i = 0; i < corners.length; i++) 
			for (int j = 0; j < corners[0].length; j++)
				corners[i][j] = new Point(0,0,0);
		
		//loop through points far enough away from edge, calculates cornerness
		for(int i = difs[0]; i < bitmap.getWidth()-difs[0]; i++)
			for(int j = difs[0]; j < bitmap.getHeight()-difs[0]; j++)
			{
				ArrayList<Integer> nearby = getNearbyPoints(i,j,bitmap);	//gets points nearby
				int total = 0;
				int currentPix = bitmap.getPixel(i,j);
				for (int nearbyPix: nearby) 		//calculates difference in color from each nearby point
				{
					int redDif = Math.abs(Color.red(currentPix) - Color.red(nearbyPix));
					int blueDif = Math.abs(Color.blue(currentPix) - Color.blue(nearbyPix));
					int greenDif = Math.abs(Color.green(currentPix) - Color.green(nearbyPix));
					int dif = redDif + blueDif + greenDif;
					total+=dif;
				}
				total = total / nearby.size();		//finds average difference in color
				corners[i][j] = new Point(i,j,total);	//adds point with difference to array
			}
		
		for (Point[] points : corners)	//gets position in pixels by multiplying by precision
			for(Point point : points)
			{
				point.x = point.x*precision;
				point.y = point.y*precision;
			}
		
		return corners;
	}
	
	private static ArrayList<Point> findLocalMax(Point[][] points)
	/*
	 * find local max in each sector that is over the minimum difference
	 */
	{
		//calc size of each sector
		int xPerSector = points.length/sectors;
		int yPerSector = points[0].length/sectors;
		
		ArrayList<Point> localMaxes = new ArrayList<Point>();
		
		for(int i = 0; i < sectors; i++)		//loop through sectors
			for(int j = 0; j < sectors; j++)
			{
				Point max = new Point(0,0,0);
				for(int x = i*xPerSector; x < (i+1)*xPerSector; x++)	//loops through points in sector
					for(int y = j*yPerSector; y < (j+1)*yPerSector; y++)
						if(points[x][y].dif > max.dif)	//finds maximum dif in sector
							max = points[x][y];
				if(max.dif >= minDif)	//if max dif high enough
					localMaxes.add(max);	//add point to list of possible corners
			}
		
		return localMaxes;
	}
	
	private static ArrayList<Point>findMaximallySuppressed(ArrayList<Point> inputCorners)
	/*
	 * makes sure no corners are too close to eachother
	 */
	{
		ArrayList<Point> corners = new ArrayList<Point>(inputCorners);
		ArrayList<Point> remove = new ArrayList<Point>();
		
		remove.clear();
		for (Point point1 : corners) 	//loop through corners
		{
			if(!remove.contains(point1))
			{
				Point max = point1;
				for (Point point2 : corners) 	//loop through corners
				{
					if((point1.x != point2.x || point1.y != point2.y) && Math.hypot(point1.x-point2.x,point1.y-point2.y) < acceptableDistance)
							//if corner too close
					{
						if(point2.dif > max.dif)	//eliminate smaller corner
						{
							remove.add(max);
							max = point2;
						}
						else
							remove.add(point2);
					}
				}
			}
		}
		corners.removeAll(remove);	//remove smaller corners
		
		return corners;
	}
	
	private static ArrayList<Integer> getNearbyPoints(int x, int y, Bitmap bitmap)
	/*
	 * gets the points surrounding passed location at each given distance
	 */
	{
		
		ArrayList<Integer> nearby = new ArrayList<Integer>();
		
		for (int dif : difs) 	//loops through distances
		{
			nearby.addAll(surrounding(x,y,bitmap,dif));	//gets points
		}
		
		return nearby;
	}
	
	private static ArrayList<Integer> surrounding(int x, int y, Bitmap bitmap, int difference)
	/*
	 * finds the eight surrounding points (up-left, up, up-right, left, right, down-left, down, down-right) at a given distance
	 */
	{
		ArrayList<Integer> surround = new ArrayList<Integer>();
		
		surround.add(bitmap.getPixel(x-difference,y-difference));
		surround.add(bitmap.getPixel(x-difference,y));
		surround.add(bitmap.getPixel(x-difference,y+difference));
		
		surround.add(bitmap.getPixel(x,y-difference));
		surround.add(bitmap.getPixel(x,y+difference));
		
		surround.add(bitmap.getPixel(x+difference,y-difference));
		surround.add(bitmap.getPixel(x+difference,y));
		surround.add(bitmap.getPixel(x+difference,y+difference));
		
		return surround;
	}
}
