package ben.namespace;

//java utilities
import java.util.ArrayList;

//app level
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.provider.MediaStore;

//component level
import android.view.Display;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

//graphics level
import android.graphics.Bitmap;

public class HelloWorldActivity extends Activity implements OnTouchListener
{
	//intent codes
	private final int TAKE_PIC_CODE = 00001;
	
	//dimensions of window
	private int x;
	private int y;
	
	//logic object
	private Logic logic;
	
	//graphical elements
	private DrawingOverlay drawOverlay;
	private Bitmap bitmap;
	
	//start up
    @Override
    public void onCreate(Bundle savedInstanceState) 
    {
        //create instance of app
    	super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        //get dims of screen
        Display display = getWindowManager().getDefaultDisplay();
        x = display.getWidth();
        y = display.getHeight();
        
        //create overlay for geometric shapes
        drawOverlay = new DrawingOverlay(this);
        addContentView(drawOverlay, new LayoutParams(x, y));
        
        //set up logic
        logic = new Logic(this);
        
        //set button listeners
        getPhotoButton().setOnClickListener(photoListener);  
        getPointButton().setOnClickListener(pointListener);  
        getDrawButton().setOnClickListener(drawListener);  
        getLineButton().setOnClickListener(lineListener);  
        getAdjustButton().setOnClickListener(adjustListener);  
        getCalcButton().setOnClickListener(calcListener);  
        
        //set touch listener on image
        getImageView().setOnTouchListener(this);
        
        //set up message
        sendStringStatus(R.string.message_welcome);
    }
    protected void onActivityResult (int requestCode, int resultCode, Intent data)
    //called whenever returns to app from an intent
    {
    	switch(requestCode)
    	{
    		case TAKE_PIC_CODE:	//returning from taking a picture
    		{
    			recieveCameraIntent(data);
    			break;
    		}
    	}
    	
    }
    
    //reaction to touch
    @Override
	public boolean onTouch(View view, MotionEvent motion) 
/*	On touch listeners listen for many types of touch events.
	Divides into clicking and moving events*/
	{
    	switch(motion.getActionMasked())
    	{
    		case MotionEvent.ACTION_DOWN:
    		{
    			logic.clicked((int)motion.getX(),(int)motion.getY());
    			break;
    		}
    		case MotionEvent.ACTION_MOVE:
    		{
    			logic.moved((int)motion.getX(),(int)motion.getY());
    			break;
    		}
    	}

		return true;	//allows future calls from same gesture
	}
   
    //draw shapes
    public void drawCircles(ArrayList<Point> circles)
    {
    	drawOverlay.setCircles(circles);
    	drawOverlay.invalidate();
    }
    public void drawLines(ArrayList<Line> lines)
    {
    	drawOverlay.setLines(lines);
    	drawOverlay.invalidate();
    }
    public void drawPoints(ArrayList<Point> points)
    {
    	drawOverlay.setPoints(points);
    	drawOverlay.invalidate();
    }
    
    //camera
    private void sendCameraIntent()
    {  	
    	//dispatch picture taking intent
    	Intent pictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        startActivityForResult(pictureIntent, TAKE_PIC_CODE);
    }
    private void recieveCameraIntent(Intent intent) 
    /* On return from camera, display image taken*/
    {
        //get bitmap from intent
    	Bundle extras = intent.getExtras();
        bitmap = (Bitmap)extras.get("data");
       
        //get dims of bitmap
        int width = bitmap.getWidth();
        int height = bitmap.getHeight();
        
        //scale image so one side matches display, one side longer
        double scaleFactor = Math.max((double)x/width,(double)y/height);
        bitmap = Bitmap.createScaledBitmap(bitmap, (int)(width*scaleFactor), (int)(height*scaleFactor), false);

        //trim and center image
        int xTrim = Math.abs(x - bitmap.getWidth());
        int yTrim = Math.abs(y - bitmap.getHeight());
        bitmap = Bitmap.createBitmap(bitmap,xTrim/2,yTrim/2,x,y);
        
        //add image to image viewer component
        ImageView view = getImageView();
        view.setImageBitmap(bitmap);
    }
    
    //status message
    public void sendStatus(String text)
    /* displays a message on the text view at bottom of screen*/
    {
    	getTextView().setText(text);
    }
	public void sendStepStatus(Step step) 
	/*displays the string resource matching the value of logic.getStep()*/
	{
		sendStringStatus(getResources().getIdentifier(step.getName(), "string", getPackageName()));
	}
	public void sendStringStatus(int ID)
	/*displays the string resource matching given ID*/
	{
		sendStatus(getStringResource(ID));
	}
	public String getStringResource(int ID)
	{
		return getResources().getString(ID);
	}
    
    //button listeners
	public OnClickListener photoListener = new OnClickListener()
	// listener for photo button
	{
		public void onClick(View v) {
			logic.resetDrawingSettings();
			
			// reset lines, corners()
			logic.clearAngle();
			logic.resetCorners();

			// dispatch picture
			sendCameraIntent();
		}
	};
	public OnClickListener pointListener = new OnClickListener()
	// listener for corner calculating button
	{
		public void onClick(View v) {
			if (bitmap != null) // if their is an image
			{
				logic.endAngleAdjusting();
				
				sendStringStatus(R.string.message_calculating); // update message
				
				// find new corners
				logic.findCorners();
				
			} else // if no image taken
				sendStringStatus(R.string.message_no_pic); // update message
		}
	};
	public OnClickListener drawListener = new OnClickListener()
	// manual drawing button
	{
		public void onClick(View v) {
			// setup angle drawing
			logic.setupAngleDrawing(false);
		}
	};
	public OnClickListener lineListener = new OnClickListener()
	// automatic drawing button
	{
		public void onClick(View v) {
				
			if(logic.areCornersPresent())
			{
				// setup angle drawing
				logic.setupAngleDrawing(true);
			}
			else
				sendStringStatus(R.string.message_no_corners);
		}
	};
	public OnClickListener adjustListener = new OnClickListener()
	// automatic drawing button
	{
		public void onClick(View v) {
			logic.setupAngleAdjusting();
			sendStringStatus(R.string.message_move);
		}
	};
	public OnClickListener calcListener = new OnClickListener()
	// calculate angle button
	{
		public void onClick(View v) {
			logic.endAngleAdjusting();
			logic.calcAngles();
		}
	};

	//retrieve component by ID
	
	//component retrieval
	private Button getPhotoButton()
	{
		return (Button)findViewById(R.id.photoButton);
	}
	private Button getPointButton()
	{
		return (Button)findViewById(R.id.pointButton);
	}
	private Button getDrawButton()
	{
		return (Button)findViewById(R.id.drawButton);
	}
	private Button getLineButton()
	{
		return (Button)findViewById(R.id.lineButton);
	}
	private Button getAdjustButton()
	{
		return (Button)findViewById(R.id.adjustButton);
	}
	private Button getCalcButton()
	{
		return (Button)findViewById(R.id.calcButton);
	}
	private ImageView getImageView()
	{
		return (ImageView)findViewById(R.id.view);
	}
	private TextView getTextView()
	{
		return (TextView)findViewById(R.id.text);
	}

	//needed get methods
	public int getButtonHeight()
	{
		return getPhotoButton().getHeight();
	}
	public Bitmap getBitmap()
	{
		return bitmap;
	}
}